#ifndef __IUF_EXP
#define __IUF_EXP
/** This class represent an explosion.
Version History {\small 
\begin{itemize}
\item 0.1 First Release (e.g. written only the defs without implementing or using it...)
\item 0.3 First concretely working release
\item 0.3s Added light management
\end{itemize}
Note that this class is very similar to bullet (same ttl). 
Probably in next versions we will subclass Exp and Bullet 
from a intermediate {\em Timed} class.
}
*/
class Exp : public GameObj
{
public:
	/// Minimal Constructor 
	Exp(){
		ttl=1000;
		}

	/** Time To Live (expressed in msec).
	Same of the bullet...
	*/
		int ttl;
/** It draw a simple disk.
The disk is drawn with a triangular fan transparent in the center
and yellow on the border. Note that to draw with transparency
we have to enable the blending. We have choosen to enable the 
blending only here for performance reasons. 
To avoid that other objects cover an explosions we disable 
the depth test and draw the explosion 
(that are contained in the generic ActiveObj list of the 
\Ref{GameSession}) as the last objects.
*/
	void Draw();


	/** Who is the owner of the three explosion lights.
	This is a static class member variable, therefore it is shared
	between all the instances of this class.
	*/
	static Exp* LightOwner[3];

	/** Initially all the lights are off;
	This is a static class member function therefore it can be called
	also without any class object instance (it is called in GameSession::Start())
	*/
	static void LightOffAll(){
		for(int i=0;i<3;++i)
				LightOwner[i]=0;
		}

  
	/** Find a free light and enable it with the current position of the explosion.
	It store the "this" pointer into the LightOwner vector.
	*/
	void LightOn()
		{
			int i =0;
			while(LightOwner[i] && i<3) ++i;

			if(i!=3) 
			{
				LightOwner[i]=this;
				float lp[4]={p.v[0],p.v[1],p.v[2],1};
				glPushMatrix();
				glLoadIdentity();
				glLightfv(GL_LIGHT1+i,GL_POSITION,lp);
				glPopMatrix();
				glEnable(GL_LIGHT1+i);
			}
		}

	/** Turn off the light and free it.
	Only if there were a light allocated by this object.
	*/
	void LightOff()
		{
			int i =0;
			while(LightOwner[i]!=this && i<3) ++i;
			if(i!=3) 
			{
				LightOwner[i]=0;
				glDisable(GL_LIGHT1+i);
			}

	}
	/** It must update and check the remaining Time To Live (TTL) of the bullet 
	and disactivating it if necessary. Same code of bullet...*/
	void Update(int t)
	{
		ttl-=t;
		if(ttl<0) {
			active=false;
			LightOff();
		}
		GameObj::Update(t);
	}
	
	};

#endif